@@ -66,7 +66,7 @@ UR::Object::Type->define(
     ]
 );
 
-our $VERSION = "0.42_03"; # UR $VERSION;;
+our $VERSION = "0.44_01"; # UR $VERSION;;
 
 # Borrow from the util package.
 # This will go away with refactoring.
@@ -164,16 +164,56 @@ sub is_subset_of {
     }
 
     my %operators = map { $_ => $self->operator_for($_) } $self->_property_names;
-    my $operators_match = 1;
+    my $is_subset = 1;
     foreach my $prop ( $other_template->_property_names ) {
         unless (exists $operators{$prop}) {
-            $operators_match = 0;
+            $is_subset = 0;
             last;
         }
-        $operators_match = undef if ($operators{$prop} ne $other_template->operator_for($prop));
+        $is_subset = undef if ($operators{$prop} ne $other_template->operator_for($prop));
     }
-    $cached_subset_data->{$other_template_id} = $operators_match;
-    return $operators_match;
+
+    if ($is_subset) {
+        $is_subset = $self->_is_subset_of_limit_offset($other_template);
+    }
+
+    return $cached_subset_data->{$other_template_id} = $is_subset;
+}
+
+sub _is_subset_of_limit_offset {
+    my($self, $other_template) = @_;
+
+    return 1 unless ($self->offset or defined($self->limit)
+                    or $other_template->offset or defined($other_template->limit));
+
+    # need to do a more comprehensive filter match.  If one or both templates
+    # has -limit and/or -offset, then the filters on both templates must match
+    # exactly.  Otherwise, one result set could include objects that were
+    # skipped because of the other's offset or limit
+    my @my_filters = map { $_ . $self->operator_for($_) } $self->_property_names;
+    my @other_filters = map { $_ . $other_template->operator_for($_) } $other_template->_property_names;
+    my($both, $only_my, $only_other) = UR::Util::intersect_lists(\@my_filters, \@other_filters);
+    return undef if (@$only_my or @$only_other);
+
+    my $my_offset = $self->offset || 0;
+    my $my_limit = $self->limit;
+    my $other_offset = $other_template->offset || 0;
+    my $other_limit = $other_template->limit;
+
+    my $is_subset;
+    if (defined($my_limit) and defined($other_limit)) {
+        my $my_last = $my_offset + $my_limit;
+        my $other_last = $other_offset + $other_limit;
+
+        $is_subset = ($my_offset >= $other_offset) && ($my_last <= $other_last);
+
+    } elsif (!defined($my_limit) and defined($other_limit)) {
+        $is_subset = 0;
+
+    } else {
+        $is_subset = $my_offset >= $other_offset;
+    }
+    return $is_subset;
 }
 
 
@@ -212,7 +252,7 @@ sub is_unique {
                 my $property_set = (ref($property_set) ? $property_set : [$property_set]);
                 my @properties_used_from_constraint =  
                     grep { defined($_) } 
-                    (ref($property_set) ? @$property_meta_hash{@$property_set} : $property_meta_hash->{$property_set});
+                    @$property_meta_hash{@$property_set};
                     
                 if (@properties_used_from_constraint == @$property_set) {
                     # filter imprecise operators
@@ -255,7 +295,7 @@ sub get_normalized_template_equivalent {
 
 sub get_rule_for_values {
     my $self = shift;
-    my $value_id = UR::BoolExpr::Util->values_to_value_id(@_);    
+    my $value_id = UR::BoolExpr::Util::values_to_value_id(@_);
     my $rule_id = UR::BoolExpr->__meta__->resolve_composite_id_from_ordered_values($self->id,$value_id);
     my $r = UR::BoolExpr->get($rule_id);
 #
@@ -412,8 +452,6 @@ sub sub_classify {
 
 # flyweight constructor
 # NOTE: this caches outside of the regular system since these are stateless objects
-# NOTE: It's not possible to use this to construct a template with meta-props, like
-# -hints or -order.  To do that, it'll have to also accept constant values as an arg
 sub get_by_subject_class_name_logic_type_and_logic_detail {
     my $class = shift;
     my $subject_class_name = shift;
@@ -421,18 +459,29 @@ sub get_by_subject_class_name_logic_type_and_logic_detail {
                     . ( defined($subject_class_name) ? "'$subject_class_name'" : "(undef)" ) ) unless ($subject_class_name);
     my $logic_type = shift;
     my $logic_detail = shift;
+    my $constant_value_id = shift || UR::BoolExpr::Util::values_to_value_id(); # default is an empty list of values
 
-    my $constant_value_id = UR::BoolExpr::Util->values_to_value_id(); # intentionally an empty list of values
     return $class->get(join('/',$subject_class_name,$logic_type,$logic_detail,$constant_value_id));
 }
 
-
 # The analogue of resolve in UR::BoolExpr.  @params_list is a list if
 # strings containing properties and operators separated by a space.  For ex: "some_param ="
 sub resolve {
     my($class,$subject_class_name, @params_list) = @_;
 
-    return $class->get_by_subject_class_name_logic_type_and_logic_detail($subject_class_name, "And", join(',',@params_list));
+    my(@params, @constant_values);
+    for (my $i = 0; $i < @params_list; $i++) {
+        push @params, $params_list[$i];
+        if (UR::BoolExpr::Util::is_meta_param($params_list[$i])) {
+            push @constant_values, $params_list[++$i];
+        }
+    }
+
+    return $class->get_by_subject_class_name_logic_type_and_logic_detail(
+                        $subject_class_name,
+                        "And",
+                        join(',',@params),
+                        UR::BoolExpr::Util::values_to_value_id(@constant_values));
 }
 
 sub get {
@@ -463,7 +512,8 @@ sub get {
     if ($logic_type eq "And") {
         # TODO: move into subclass
         my @keys = split(/,/,$logic_detail || '');    
-        my @constant_values = UR::BoolExpr::Util->value_id_to_values($constant_value_id) if defined $constant_value_id;;
+        my @constant_values;
+        @constant_values = UR::BoolExpr::Util::value_id_to_values($constant_value_id) if defined $constant_value_id;
         return $sub_class_name->_fast_construct(
             $subject_class_name,
             \@keys,
