@@ -6,7 +6,7 @@ package UR::BoolExpr::Util;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 use Scalar::Util qw(blessed reftype refaddr);
 use Data::Dumper;
@@ -26,19 +26,39 @@ our $empty_list = chr(20);      # used for []
 # These are used when there is any sort of complicated data in the rule.
 
 sub values_to_value_id_frozen {
-    my $self = shift;
     my $frozen = FreezeThaw::safeFreeze(@_);
     return "F:" . $frozen;
 }
 
 sub value_id_to_values_frozen {
-    my $self = shift;
     my $value_id = shift;
-    return $self->_fixup_ur_objects_from_thawed_data(FreezeThaw::thaw($value_id));
+    no warnings 'redefine';
+    local *FreezeThaw::copyContents = \&_FreezeThaw__copyContents;
+    return _fixup_ur_objects_from_thawed_data(FreezeThaw::thaw($value_id));
+}
+
+# FreezeThaw::thaw calls copyContents when thawing out a data structure it's seen before to
+# copy the contents from an already thawed version into a to-be-thawed container.
+# But this doesn't work for code references, since it can't reconstitute it.  Luckily, if
+# it's been frozen with safeFreeze, then $first and $second both point to the correct
+# coderef and there's nothing to do.
+my $original_FreezeThaw__copyContents = \&FreezeThaw::copyContents;
+sub _FreezeThaw__copyContents {
+    my($first, $second) = @_;
+
+    goto &$original_FreezeThaw__copyContents if (reftype($first) ne 'CODE' or reftype($second) ne 'CODE');
+
+    if (refaddr($first) != refaddr($second)) {
+        my $ref = reftype($second);
+        Carp::croak("Don't know how to copyContents of type `$ref'");
+    }
+    if (ref($second) ne ref($first)) {
+        bless $_[0], ref $second; # Rebless
+    }
+    return $first;
 }
 
 sub _fixup_ur_objects_from_thawed_data {
-    my $self = shift;
     my @values = @_;
 
     our $seen;
@@ -71,7 +91,7 @@ sub _fixup_ur_objects_from_thawed_data {
             }
 
         }
-        $self->_fixup_ur_objects_from_thawed_data($_);
+        _fixup_ur_objects_from_thawed_data($_);
     };
 
     foreach my $data ( @values ) {
@@ -102,7 +122,6 @@ sub _fixup_ur_objects_from_thawed_data {
 # These are used for the simple common-case rules.
 
 sub values_to_value_id {
-    my $self = shift;
     my $value_id = "O:";
 
     for my $value (@_) {
@@ -123,7 +142,7 @@ sub values_to_value_id {
                         }
                         else {
                             if (ref($value2) or index($value2, $unit_sep) >= 0 or index($value2, $record_sep) >= 0) {
-                                return $self->values_to_value_id_frozen(@_);
+                                return values_to_value_id_frozen(@_);
                             }
                             $value_id .= $value2 . $unit_sep;
                         }
@@ -133,7 +152,7 @@ sub values_to_value_id {
             }
             else {
                 if (ref($value) or index($value,$unit_sep) >= 0 or index($value,$record_sep) >= 0) {
-                    return $self->values_to_value_id_frozen(@_);
+                    return values_to_value_id_frozen(@_);
                 }
                 $value_id .= $value . $record_sep;
             }
@@ -148,7 +167,6 @@ sub values_to_value_id {
 }
 
 sub value_id_to_values {
-    my $self = shift;
     my $value_id = shift;
 
     unless (defined $value_id) {
@@ -158,7 +176,7 @@ sub value_id_to_values {
     my $method_identifier = substr($value_id,0,2);
     $value_id = substr($value_id, 2, length($value_id)-2);    
     if ($method_identifier eq "F:") {
-        return $self->value_id_to_values_frozen($value_id);
+        return value_id_to_values_frozen($value_id);
     }
 
     my @values = ($value_id =~ /(.*?)$record_sep/gs);
@@ -189,7 +207,10 @@ sub value_id_to_values {
     return @values;
 }
 
-*values_to_value_id_simple = \&values_to_value_id;
+sub is_meta_param {
+    my $param_name = shift;
+    return substr($param_name, 0, 1) eq '-';
+}
 
 package UR::BoolExpr::Util::clonedThing;
 
