@@ -6,12 +6,18 @@ package UR::DataSource::Meta;
 use strict;
 use warnings;
 
+use version;
+use DBD::SQLite;
+
 use UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::Meta',
     is => ['UR::DataSource::SQLite'],
+    has_constant => [
+        owner => { value => 'main' },
+    ],
 );
 
 sub _resolve_class_name_for_table_name_fixups {
@@ -34,7 +40,7 @@ CREATE TABLE IF NOT EXISTS dd_bitmap_index (
     owner varchar,
     table_name varchar NOT NULL,
     bitmap_index_name varchar NOT NULL,
-    PRIMARY KEY (data_source, owner, table_name, bitmap_index_name)
+    PRIMARY KEY (data_source, table_name, bitmap_index_name)
 );
 CREATE TABLE IF NOT EXISTS dd_fk_constraint (
     data_source varchar NOT NULL,
@@ -44,7 +50,7 @@ CREATE TABLE IF NOT EXISTS dd_fk_constraint (
     r_table_name varchar NOT NULL,
     fk_constraint_name varchar NOT NULL,
     last_object_revision timestamp NOT NULL,
-    PRIMARY KEY(data_source, owner, r_owner, table_name, r_table_name, fk_constraint_name)
+    PRIMARY KEY(data_source, table_name, r_table_name, fk_constraint_name)
 );
 CREATE TABLE IF NOT EXISTS dd_fk_constraint_column (
     fk_constraint_name varchar NOT NULL,
@@ -55,7 +61,7 @@ CREATE TABLE IF NOT EXISTS dd_fk_constraint_column (
     column_name varchar NOT NULL,
     r_column_name varchar NOT NULL,
 
-    PRIMARY KEY(data_source, owner, table_name, fk_constraint_name, column_name)
+    PRIMARY KEY(data_source, table_name, fk_constraint_name, column_name)
 );
 CREATE TABLE IF NOT EXISTS dd_pk_constraint_column (
     data_source varchar NOT NULL,
@@ -63,7 +69,7 @@ CREATE TABLE IF NOT EXISTS dd_pk_constraint_column (
     table_name varchar NOT NULL,
     column_name varchar NOT NULL,
     rank integer NOT NULL,
-    PRIMARY KEY (data_source,owner,table_name,column_name,rank)
+    PRIMARY KEY (data_source,table_name,column_name,rank)
 );
 CREATE TABLE IF NOT EXISTS dd_table (
      data_source varchar NOT NULL,
@@ -74,7 +80,7 @@ CREATE TABLE IF NOT EXISTS dd_table (
      last_ddl_time timestamp,
      last_object_revision timestamp NOT NULL,
      remarks varchar,
-     PRIMARY KEY(data_source, owner, table_name)
+     PRIMARY KEY(data_source, table_name)
 );
 CREATE TABLE IF NOT EXISTS dd_table_column (
     data_source varchar NOT NULL,
@@ -86,7 +92,7 @@ CREATE TABLE IF NOT EXISTS dd_table_column (
     nullable varchar NOT NULL,
     last_object_revision timestamp NOT NULL,
     remarks varchar,
-    PRIMARY KEY(data_source, owner, table_name, column_name)
+    PRIMARY KEY(data_source, table_name, column_name)
 );
 CREATE TABLE IF NOT EXISTS dd_unique_constraint_column (
     data_source varchar NOT NULL,
@@ -94,7 +100,7 @@ CREATE TABLE IF NOT EXISTS dd_unique_constraint_column (
     table_name varchar NOT NULL,
     constraint_name varchar NOT NULL,
     column_name varchar NOT NULL,
-    PRIMARY KEY (data_source,owner,table_name,constraint_name,column_name)
+    PRIMARY KEY (data_source,table_name,constraint_name,column_name)
 );
 EOS
 
@@ -158,6 +164,19 @@ sub generate_for_namespace {
     return ($meta_datasource, $meta_db_file);
 }
 
+sub _dbi_connect_args {
+    my $self = shift;
+
+    my @connection = $self->SUPER::_dbi_connect_args(@_);
+
+    if(version->parse($DBD::SQLite::VERSION) >= version->parse('1.38_01')) {
+        my $connect_attr = $connection[3] ||= {};
+        $connect_attr->{sqlite_use_immediate_transaction} = 0;
+    }
+
+    return @connection;
+}
+
 1;
 
 =pod
