@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::Oracle',
@@ -17,6 +17,8 @@ sub owner { shift->_singleton_object->login }
 
 sub can_savepoint { 1 }  # Oracle supports savepoints inside transactions
 
+sub does_support_limit_offset { 0 }
+
 sub does_support_recursive_queries { 'connect by' };
 
 sub set_savepoint {
@@ -226,7 +228,7 @@ sub set_userenv {
 sub get_userenv {
 
     # there are two ways to set these values but this is
-    # the only way to retreive the values after they are set
+    # the only way to retrieve the values after they are set
 
     my ($self, $dbh) = @_;
 
