@@ -3,7 +3,7 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::DataSource::Pg',
@@ -140,8 +140,11 @@ my %ur_data_type_for_vendor_data_type = (
      'SMALLINT'  => ['Integer', undef],
      'BIGINT'    => ['Integer', undef],
      'SERIAL'    => ['Integer', undef],
-     'TEXT'      => ['XmlBlob', undef],
+     'TEXT'      => ['Text', undef],
      'BYTEA'     => ['Blob', undef],
+     'CHARACTER VARYING' => ['Text', undef],
+     'TIMESTAMP WITHOUT TIME ZONE' => ['DateTime', undef],
+     'NUMERIC'   => ['Number', undef],
 
      'DOUBLE PRECISION' => ['Number', undef],
 );
@@ -219,16 +222,38 @@ sub cast_for_data_conversion {
 sub _resolve_order_by_clause_for_column {
     my($self, $column_name, $query_plan, $property_meta) = @_;
 
-    my $column_clause = $self->SUPER::_resolve_order_by_clause_for_column($column_name, $query_plan);
+    my $column_clause = $column_name;
 
     my $is_text_type = $property_meta->is_text;
     if ($is_text_type) {
         # Tell the DB to sort the same order as Perl's cmp
         $column_clause .= q( COLLATE "C");
     }
+
+    my $is_desc = $query_plan->order_by_column_is_descending($column_name);
+    if ($is_desc) {
+        $column_clause .= q( DESC);
+    }
+
     return $column_clause;
 }
 
+sub _assure_schema_exists_for_table {
+    my($self, $table_name, $dbh) = @_;
+
+    $dbh ||= $self->get_default_handle;
+
+    my ($schema_name, undef) = $self->_extract_schema_and_table_name($table_name);
+    if ($schema_name) {
+        my $exists = $dbh->selectrow_array("SELECT schema_name FROM information_schema.schemata WHERE schema_name = ?;",
+            undef, $schema_name);
+        unless ($exists) {
+            $dbh->do("CREATE SCHEMA $schema_name")
+                or Carp::croak("Could not create schema $schema_name: " . $dbh->errstr);
+        }
+    }
+}
+
 1;
 
 =pod
