@@ -4,8 +4,9 @@ use strict;
 use warnings;
 
 use UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 use IO::File;
+use POSIX qw(ENOENT);
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
@@ -48,50 +49,65 @@ sub is_sub_command_delegator { 0; }
 sub execute {
     my $self = shift;
     my $class = $self->classname;
-   
-    eval {
-        require Getopt::Complete;
-        require Getopt::Complete::Cache;
+
+    my $req_exception = do {
+        local $@;
+        eval {
+            require Getopt::Complete;
+            require Getopt::Complete::Cache;
+        };
+        $@;
     };
-    if ($@) {
+    if ($req_exception) {
         die "Errors using Getopt::Complete.  Do you have Getopt::Complete installed?  If not try 'cpanm Getopt::Complete'";
     }
 
-    eval "use above '$class';";
-    if ($@) {
+    my $use_exception = do {
+        local $@;
+        eval "use above '$class';";
+        $@;
+    };
+    if ($use_exception) {
         $self->error_message("Unable to use above $class.\n$@");
         return;
     }
 
     (my $module_path) = Getopt::Complete::Cache->module_and_cache_paths_for_package($class, 1);
     my $cache_path = $module_path . ".opts";
-    if (-s $cache_path) {
-        rename($cache_path, "$cache_path.bak");
-    }
-    unless ($self->output) {
-        $self->output($cache_path);
-    }
-    $self->status_message("Generating " . $self->output . " file for $class.");
-    $self->status_message("This may take some time and may generate harmless warnings...");
 
-    my $fh;
-    $fh = IO::File->new('>' . $self->output) || die "Cannot create file at " . $self->output . "\n";
-    
-    if ($fh) {
+    eval {
+        my $rename_ok = rename($cache_path, "$cache_path.bak");
+        if (!$rename_ok && $! != ENOENT) {
+            die "failed to rename file: $!: $cache_path";
+        }
+
+        unless ($self->output) {
+            $self->output($cache_path);
+        }
+        $self->status_message("Generating " . $self->output . " file for $class.");
+        $self->status_message("This may take some time and may generate harmless warnings...");
+
+        my $fh = IO::File->new($self->output, 'w')
+            or die "Cannot create file at " . $self->output . "\n";
+
         my $src = Data::Dumper::Dumper($class->resolve_option_completion_spec());
         $src =~ s/^\$VAR1/\$$class\:\:OPTS_SPEC/;
         $fh->print($src);
-    }
+    };
+
     if (-s $cache_path) {
         $self->status_message("\nOPTS_SPEC file created at $cache_path");
-        unlink("$cache_path.bak");
+        unlink("$cache_path.bak")
+            or $self->error_message("failed to remove backup file: $!");
     } else {
         if (-s "$cache_path.bak") {
             $self->error_message("$cache_path is 0 bytes, reverting to previous");
-            rename("$cache_path.bak", $cache_path);
+            rename("$cache_path.bak", $cache_path)
+                or $self->error_message("failed to restore file: $!");
         } else {
             $self->error_message("$cache_path is 0 bytes and no backup exists, removing file");
-            unlink($cache_path);
+            unlink($cache_path)
+                or $self->error_message("failed to remove file: $!");
         }
     }
 }
