@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use UR;
 use List::MoreUtils qw(any);
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 our @CARP_NOT = qw( UR::Object::Type );
 
@@ -13,6 +13,7 @@ use overload ('==' => sub { $_[0] . ''  eq $_[1] . '' } );
 use overload ('eq' => sub { $_[0] . ''  eq $_[1] . '' } );
 use overload ('!=' => sub { $_[0] . ''  ne $_[1] . '' } );
 use overload ('ne' => sub { $_[0] . ''  ne $_[1] . '' } );
+use overload ('cmp' => sub { $_[0]->id cmp $_[1]->id } );
 
 class UR::Object::Set {
     is => 'UR::Value',
@@ -122,13 +123,9 @@ sub get_with_special_parameters {
     my $class = shift;
     my $bx = shift;
     my @params = @_;
-
     my $member_class = $class;
     $member_class =~ s/::Set$//;
-
-    my $rule = UR::BoolExpr->resolve($member_class, $bx->params_list, @params);
-
-    return $class->get($rule->id);
+    return $member_class->define_set($bx->params_list, @params);
 }
 
 sub members {
@@ -152,8 +149,10 @@ sub member_iterator {
 sub _members_have_changes {
     my $self = shift;
     return 1 if $self->{__members_have_changes};
+
+    my @property_names = @_;
     my $rule = $self->rule;
-    return any { $rule->evaluate($_) && $_->__changes__(@_) } $self->member_class_name->is_loaded;
+    return any { $rule->evaluate($_) && $_->__changes__(@property_names) } $self->member_class_name->is_loaded;
 }
 
 sub subset {
