@@ -3,7 +3,7 @@ use warnings;
 use strict;
 
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 use Sys::Hostname;
 use Cwd;
@@ -36,7 +36,9 @@ sub data_source {
     my $ds = $self->data_source_id(@_);
     
     return undef unless $ds;
-    my $obj = UR::DataSource->get($ds) || $ds->get();
+    local $@;
+    my $obj = eval { UR::DataSource->get($ds) || $ds->get() };
+
     return $obj;
 }
 
@@ -207,11 +209,16 @@ sub resolve_property_aliases {
             # there was a problem resolving the chain of properties
             # This happens in the case of an object accessor (is => 'Some::Class') without an id_by
             my @split_names = split(/\./,$property_name);
+            my $name_count = @split_names;
             my $prop_meta = $self->property_meta_for_name(shift @split_names);
             return unless $prop_meta;
             my $foreign_class = $prop_meta->data_type && eval { $prop_meta->data_type->__meta__};
             return unless $foreign_class;
             @property_names = ( $prop_meta->alias_for, $foreign_class->resolve_property_aliases(join('.', @split_names)));
+            unless (@property_names >= $name_count) {
+                Carp::croak("Some parts from property '$property_name' of class ".$self->class_name
+                            . " didn't resolve");
+            }
         }
         $self->{'_resolve_property_aliases'}->{$property_name} = join('.', @property_names);
     }
@@ -583,9 +590,6 @@ sub id_property_sorter {
 }
 
 sub sorter {
-    #TODO: make this take +/- indications of ascending/descending
-    #TODO: make it into a closure for speed
-    #TODO: there are possibilities of it sorting different than a DB on mixed numbers and alpha data
     my ($self,@properties) = @_;
     push @properties, $self->id_property_names;
     my $key = join("__",@properties);
@@ -601,19 +605,10 @@ sub sorter {
                 push @is_descending, 0;
             }
 
-            my $class_meta;
-            if ($self->isa("UR::Object::Set::Type")) {
-                # If we're a set, we want to examine the property of our members.
-                my $subject_class = $self->class_name;
-                $subject_class =~ s/::Set$//g;
-                $class_meta = $subject_class->__meta__;#->property($property);
-            } else {
-                $class_meta = $self;
-            }
-
-            my ($pmeta,@extra) = $class_meta->_concrete_property_meta_for_class_and_name($property);
+            my ($pmeta,@extra) = $self->_concrete_property_meta_for_class_and_name($property);
             if(@extra) {
-                $pmeta = $class_meta->property($property); #a composite property (typically ID)
+                # maybe a composite property (typically ID), or a chained property (prop.other_prop)
+                $pmeta = $self->property_meta_for_name($property);
             }
 
             if ($pmeta) {
@@ -629,7 +624,7 @@ sub sorter {
             }
         }
 
-        no warnings;   # don't print a warning about undef values ...alow them to be treated as 0 or '' 
+        no warnings 'uninitialized';
         $sorter = $self->{_sorter}{$key} ||= sub($$) {
 
             for (my $n = 0; $n < @properties; $n++) {
@@ -861,6 +856,7 @@ sub generate_support_class_for_extension {
         delete $class_params{id_generator};
         delete $class_params{id};
         delete $class_params{is};
+        delete $class_params{roles};
 
         my $attributes_have = UR::Util::deep_copy($subject_class_obj->{attributes_have});
         my $class_props = UR::Util::deep_copy($subject_class_obj->{has});    
@@ -900,13 +896,24 @@ sub has_table {
     for my $class_name (@parent_classes) {
         next if $class_name eq "UR::Object";
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        if ($class_obj->table_name) {
+        if ($class_obj->has_direct_table) {
             return 1;
         }
     }
     return;
 }
 
+sub has_direct_table {
+    my $self = shift;
+    return 1 if $self->table_name;
+
+    if ($self->data_source_id and $self->data_source_id->isa('UR::DataSource::Default')) {
+        my $load_function_name = join('::', $self->class_name, '__load__');
+        return 1 if exists &$load_function_name;
+    }
+    return;
+}
+
 sub most_specific_subclass_with_table {
     my $self = shift;
 
@@ -914,7 +921,7 @@ sub most_specific_subclass_with_table {
 
     foreach my $class_name ( $self->class_name->inheritance ) {
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        return $class_name if ($class_obj && $class_obj->table_name);
+        return $class_name if ($class_obj and $class_obj->has_direct_table);
     }
     return;
 }
@@ -925,7 +932,7 @@ sub most_general_subclass_with_table {
     my @subclass_list = reverse ( $self->class_name, $self->class_name->inheritance );
     foreach my $class_name ( $self->inheritance ) {
         my $class_obj = UR::Object::Type->get(class_name => $class_name);
-        return $class_name if ($class_obj && $class_obj->table_name);
+        return $class_name if ($class_obj && $class_obj->has_direct_table);
     }
     return;
 }
@@ -994,20 +1001,25 @@ sub _load {
     }
 
     # Check the filesystem.  The file may create its metadata object.
-    eval "use $class_name";
-    unless ($@) {
+    my $exception = do {
+        local $@;
+        eval "use $class_name";
+        $@;
+    };
+    unless ($exception) {
         # If the above module was loaded, and is an UR::Object,
         # this will find the object.  If not, it will return nothing.
         $class_obj = $UR::Context::current->get_objects_for_class_and_rule($class,$rule,0);
         return $class_obj if $class_obj;
     }
-    if ($@) {
+    if ($exception) {
         # We need to handle $@ here otherwise we'll see
         # "Can't locate UR/Object/Type/Ghost.pm in @INC" error.
         # We want to fall through "in the right circumstances".
         (my $module_path = $class_name . '.pm') =~ s/::/\//g;
-        Carp::croak("Error while autoloading with 'use $class_name': $@") unless ($@ =~ /Can't locate $module_path in \@INC/);
-        # FIXME: I think other conditions here will result in silent errors.
+        unless ($exception =~ /Can't locate $module_path in \@INC/) {
+            die "Error while autoloading with 'use $class_name': $exception";
+        }
     }
 
     # Parse the specified class name to check for a suffix.
@@ -1035,7 +1047,15 @@ sub _load {
         # which would fire recursively for three extensions of
         # Acme::Equipment.
         my $full_base_class_name = $prefix . ($base ? "::" . $base : "");
-        my $base_class_obj = eval { $full_base_class_name->__meta__ };
+        my $base_class_obj;
+        my $exception = do {
+            local $@;
+            $base_class_obj = eval { $full_base_class_name->__meta__ };
+            $@;
+        };
+        if ($exception && $exception =~ m/^Error while autoloading/) {
+            die $exception;
+        }
 
         if ($base_class_obj)
         {
@@ -1438,6 +1458,7 @@ sub _property_change_callback {
     my($property_obj,$method, $old_val, $new_val) = @_;
 
     return if ($method eq 'load' || $method eq 'unload');
+    return unless ref($property_obj);  # happens when, say, error_message is called on the UR::Object::Property class
 
     my $class_obj = UR::Object::Type->get(class_name => $property_obj->class_name);
     my $property_name = $property_obj->property_name;
@@ -1470,7 +1491,8 @@ sub _property_change_callback {
         &_id_property_change_callback($property_obj, $change);
     }
 
-    if (exists $class_obj->{'has'}->{$property_name}->{$method}) {
+    if (exists $class_obj->{'has'}->{$property_name}
+        && exists $class_obj->{'has'}->{$property_name}->{$method}) {
         $class_obj->{'has'}->{$property_name}->{$method} = $new_val;
 
     } 
@@ -1575,19 +1597,15 @@ sub __signal_change__ {
     return @rv;
 }
 
-our %STANDARD_VALID_SIGNALS = ( create        => 1,
-                                'delete'      => 1,
-                                commit        => 1,
-                                rollback      => 1,
-                                load          => 1,
-                                unload        => 1,
-                                load_external => 1 );
+my @default_valid_signals = qw(create delete commit rollback load unload load_external subclass_loaded);
+our %STANDARD_VALID_SIGNALS;
+@STANDARD_VALID_SIGNALS{@default_valid_signals} = (1) x @default_valid_signals;
 sub _is_valid_signal {
     my $self = shift;
     my $aspect = shift;
 
-    # Undefined attributes indicate that the subscriber wants any changes at all to generate a callback.
-    return 1 if (! defined $aspect);
+    # An aspect of empty string (or undef) means all aspects are being observed.
+    return 1 unless (defined($aspect) and length($aspect));
 
     # All standard creation and destruction methods emit a signal.
     return 1 if ($STANDARD_VALID_SIGNALS{$aspect});
@@ -1642,6 +1660,67 @@ sub ungenerate {
     delete $INC{$module_name};    
     $self->{'generated'} = 0;
 }
-    
+
+sub singular_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+    unless (exists $self->{_accessor_singular_names}->{$property_name}) {
+        my $property_meta = $self->property_meta_for_name($property_name) if ($self->generated);
+        if ($bootstrapping  # trust the caller when bootstrapping
+            or
+            ! $self->generated # when called from UR::Object::Type::AccessorWriter and the property isn't created yet
+            or
+            ($property_meta && $property_meta->is_many)
+        ) {
+            require Lingua::EN::Inflect;
+            $self->{_accessor_singular_names}->{$property_name} = Lingua::EN::Inflect::PL_V($property_name);
+        } else {
+            $self->{_accessor_singular_names}->{$property_name} = undef;
+        }
+    }
+    return $self->{_accessor_singular_names}->{$property_name};
+}
+
+sub iterator_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_iterator";
+}
+
+sub set_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_set";
+}
+
+sub rule_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "__${singular}_rule";
+}
+
+sub arrayref_accessor_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "${singular}_arrayref";
+}
+
+sub adder_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "add_${singular}";
+}
+
+sub remover_name_for_is_many_accessor {
+    my($self, $property_name) = @_;
+
+    my $singular = $self->singular_accessor_name_for_is_many_accessor($property_name);
+    return $singular && "remove_${singular}";
+}
+
 1;
 
