@@ -8,7 +8,7 @@ require UR;
 use List::MoreUtils qw(first_index);
 use Scalar::Util qw(looks_like_number);
 
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 our %meta_classes;
 our $bootstrapping = 1;
@@ -109,7 +109,7 @@ sub resolve_class_description_perl {
 
     # These property names are either written in other places in this sub, or shouldn't be written out
     my %addl_property_names = map { $_ => 1 } $self->__meta__->all_property_type_names;
-    my @specified = qw/is class_name table_name id_by er_role is_abstract generated data_source_id schema_name doc namespace id first_sub_classification_method_name property_metas pproperty_names id_property_metas meta_class_name id_generator valid_signals/;
+    my @specified = qw/is class_name table_name id_by er_role is_abstract generated data_source_id schema_name doc namespace id first_sub_classification_method_name property_metas pproperty_names id_property_metas meta_class_name id_generator valid_signals roles/;
     delete @addl_property_names{@specified};
     for my $property_name (sort keys %addl_property_names) {
         my $property_obj = $self->__meta__->property_meta_for_name($property_name);
@@ -225,6 +225,10 @@ sub resolve_class_description_perl {
         $perl .= "    id_generator => '$id_generator',\n" if ($print_id_generator);
     }
 
+    if ($self->roles and @{ $self->roles }) {
+        $perl .= "    roles => " . pprint_arrayref($self->roles) . ",\n";
+    }
+
     if (my $valid_signals = $self->valid_signals) {
         if (ref($valid_signals) ne 'ARRAY') {
             Carp::croak("The 'valid_signals' metadata for class $class_name must be an arrayref, got: ".Data::Dumper::Dumper($valid_signals));
@@ -358,7 +362,11 @@ sub _get_display_fields_for_property {
             $via_property_name = $property->to;
         }
         my $via = $property->class_name->__meta__->properties(property_name => $via_property_name);
-        if ($property->is_many ne $via->is_many) {
+        if (! $via) {
+            # maybe via a method??  Safer to use is_many than not
+            push @fields, 'is_many => 1';
+
+        } elsif ($property->is_many ne $via->is_many) {
             push @fields, 'is_many => ' . $property->is_many;
         }
     }
@@ -749,26 +757,37 @@ sub pprint_section {
     return "$indent_section$section => [\n$section_src$indent_section],\n";
 }
 
-
-sub pprint_subsection {
-    my ($name, @fields) = @_;
+{
     my $indent_name = ' ' x 8;
     my $indent_key  = $indent_name . ' ' x 4;
+    my $max_width = 78;
+    sub pprint_subsection {
+        my ($name, @fields) = @_;
 
-    my $section_src;
-    foreach ( @fields ) { s/^\s+// }
-    if (@fields > 1) {
-        my $line =
-        $indent_name . $name . " => {\n"
-        . $indent_key . join(",\n$indent_key", @fields) . ",\n"
-        . $indent_name . "},\n";
-
-        $section_src = $line;
-    } else {
-        $section_src = $indent_name . $name . " => { " . (defined $fields[0] ? $fields[0] : '') . " },\n";
+        foreach ( @fields ) { s/^\s+// }
+        my $section_src = _pprint_subsection_one_line($name, @fields);
+        if (length($section_src) > $max_width) {
+            $section_src = _pprint_subsection_multi_line($name, @fields);
+        }
+        return $section_src;
     }
-    return $section_src;
-};
+
+    sub _pprint_subsection_one_line {
+        my $name = shift;
+
+        return $indent_name . $name . ' => { '
+                    . join(', ', @_)
+                    . " },\n";
+    }
+
+    sub _pprint_subsection_multi_line {
+        my $name = shift;
+
+        return $indent_name . $name . " => {\n"
+            . $indent_key . join(",\n$indent_key", @_) . ",\n"
+            . $indent_name . "},\n";
+    }
+}
 
 sub _quoted_value {
     my $value = shift;
