@@ -5,7 +5,7 @@ package UR::Object;
 use warnings;
 use strict;
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 
 use Data::Dumper;
 use Scalar::Util qw(blessed);
@@ -217,10 +217,20 @@ sub create_subscription  {
     # parse parameters
     my ($class,$id,$method,$callback,$note,$priority);
 
+    my %translate = (
+        method => 'aspect',
+        id => 'subject_id',
+    );
+    my @param_names = qw(method callback note priority id);
     my %observer_params;
-    @observer_params{'aspect','callback','note','priority','subject_id'} = delete @params{'method','callback','note','priority','id'};
+    for my $name (@param_names) {
+        if (exists $params{$name}) {
+            my $obs_name = $translate{$name} || $name;
+            $observer_params{$obs_name} = delete $params{$name};
+        }
+    }
+
     $observer_params{'subject_class_name'} = $self->class;
-    $observer_params{'priority'} = 1 unless defined $observer_params{'priority'};
     if (!defined $observer_params{'subject_id'} and ref($self)) {
         $observer_params{'subject_id'} = $self->id;
     }
@@ -239,35 +249,15 @@ sub create_subscription  {
     return [@observer_params{'subject_class_name','subject_id','aspect','callback','note'}];
 }
 
+
 sub validate_subscription
 {
-    return 1;
-
-    my ($self,$subscription_property) = @_;
-
-    Carp::confess("The _create_object and _delete_object signals are no longer emitted!") 
-        if defined($subscription_property) 
-            and ($subscription_property eq '_create_object' or $subscription_property eq '_delete_object');
-
-    # Undefined attributes indicate that the subscriber wants any changes at all to generate a callback.
-    return 1 if (!defined($subscription_property));
-
-    # All standard creation and destruction methods emit a signal.
-    return 1 if ($subscription_property =~ /^(create|delete|commit|rollback|load|unload|load_external)$/);
-
-    # A defined attribute in our property list indicates the caller wants callbacks from our properties.
-    my $class_object = $self->__meta__;
-    for my $property ($class_object->all_property_names)
-    {
-        return 1 if $property eq $subscription_property;
-    }
-
-    return 1 if ($class_object->_is_valid_signal($subscription_property));
-
-    # Bad subscription request.
+    # Everything is invalid unless you make it valid by implementing
+    # validate_subscription on your class.  (Or use the new API.)
     return;
 }
 
+
 sub inform_subscription_cancellation
 {
     # This can be overridden in derived classes if the class wants to know
