@@ -4,7 +4,7 @@ package UR::Util;
 use warnings;
 use strict;
 require UR;
-our $VERSION = "0.42_03"; # UR $VERSION;
+our $VERSION = "0.44_01"; # UR $VERSION;
 use Cwd;
 use Data::Dumper;
 use Clone::PP;
@@ -694,6 +694,49 @@ sub use_package_optimistically {
     return $INC{$file};
 }
 
+# return a hashref of subroutine names => coderefs
+sub coderefs_for_package {
+    my $package = shift;
+
+    my %stash = do {
+        no strict 'refs';
+        my $stash_name = $package . '::';
+        %$stash_name;
+    };
+
+    my %subs;
+    local $@;
+    foreach my $name ( keys %stash ) {
+        my $glob = $stash{$name};
+        next unless my $coderef = eval { *$glob{CODE} };  # constants are SCALAR refs, not typeglobs
+        $subs{$name} = $coderef;
+    }
+    return \%subs;
+}
+
+# Given a key in a hashref, if the value is a scalar, wrap it in an arrayref
+# used by the class initializer to allow some keys in a class definition to
+# be specified as simple scalars that are normalized to be an arrayref.
+# Returns false if the value isn't an arrayref or scalar.
+sub ensure_arrayref {
+    my($new_class, $key) = @_;
+
+    if ($new_class->{$key}) {
+        if (!ref($new_class->{$key})) {
+            # If it's a plain string, wrap it into an arrayref
+            $new_class->{$key} = [ $new_class->{$key} ];
+        } elsif (ref($new_class->{$key}) ne 'ARRAY') {
+            my $class_name = $new_class->{class_name};
+            return 0;
+        }
+    } else {
+        $new_class->{$key} = [];
+    }
+    return 1;
+}
+
+
+
 1;
 
 =pod
